Package: BiocNeighbors
Version: 2.6.0
Date: 2026-02-13
Title: Nearest Neighbor Detection for Bioconductor Packages
Authors@R: c(person("Aaron", "Lun", role=c("aut", "cre", "cph"), email="infinite.monkeys.with.keyboards@gmail.com"))
Imports: Rcpp, methods
Suggests: Matrix, DelayedArray, beachmat, BiocParallel, testthat,
        BiocStyle, knitr, rmarkdown
biocViews: Clustering, Classification
Description: 
    Implements exact and approximate methods for nearest neighbor detection, in a framework that allows them to be easily switched within Bioconductor packages or workflows.
    Exact searches can be performed using the k-means for k-nearest neighbors algorithm, vantage point trees, or an exhaustive search.
    Approximate searches can be performed using the Annoy or HNSW libraries.
    Each search can be performed with a variety of different distance metrics, parallelization, and variable numbers of neighbors.
    Range-based searches (to find all neighbors within a certain distance) are also supported.
License: GPL-3
LinkingTo: Rcpp, assorthead, beachmat
VignetteBuilder: knitr
SystemRequirements: C++17
RoxygenNote: 7.3.3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/BiocNeighbors
git_branch: RELEASE_3_23
git_last_commit: 221e565
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:54:12 UTC; biocbuild
Author: Aaron Lun [aut, cre, cph]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
