# Nearest neighbor methods for Bioconductor

|Build|Status|
|-----|----|
| Bioc-release | [![](https://bioconductor.org/shields/build/release/bioc/BiocNeighbors.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/BiocNeighbors) |
| Bioc-devel   | [![](https://bioconductor.org/shields/build/devel/bioc/BiocNeighbors.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/BiocNeighbors) | 

This package provides a consistent C++ class interface for nearest neighbor detection based on the [**knncolle**](https://github.com/knncolle/knncolle) C++ library.
Check out the user guide on the [Bioconductor landing page](https://bioconductor.org/packages/devel/bioc/html/BiocNeighbors.html) for more details.
