# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

annoy_builder <- function(num_trees, search_mult, distance) {
    .Call('_BiocNeighbors_annoy_builder', PACKAGE = 'BiocNeighbors', num_trees, search_mult, distance)
}

exhaustive_builder <- function(distance) {
    .Call('_BiocNeighbors_exhaustive_builder', PACKAGE = 'BiocNeighbors', distance)
}

find_mutual_nns <- function(left, right) {
    .Call('_BiocNeighbors_find_mutual_nns', PACKAGE = 'BiocNeighbors', left, right)
}

generic_build <- function(builder, data) {
    .Call('_BiocNeighbors_generic_build', PACKAGE = 'BiocNeighbors', builder, data)
}

generic_num_obs <- function(prebuilt_ptr) {
    .Call('_BiocNeighbors_generic_num_obs', PACKAGE = 'BiocNeighbors', prebuilt_ptr)
}

generic_find_knn <- function(prebuilt_ptr, num_neighbors, force_variable_neighbors, chosen, num_threads, last_distance_only, report_index, report_distance) {
    .Call('_BiocNeighbors_generic_find_knn', PACKAGE = 'BiocNeighbors', prebuilt_ptr, num_neighbors, force_variable_neighbors, chosen, num_threads, last_distance_only, report_index, report_distance)
}

generic_query_knn <- function(prebuilt_ptr, query, num_neighbors, force_variable_neighbors, num_threads, last_distance_only, report_index, report_distance) {
    .Call('_BiocNeighbors_generic_query_knn', PACKAGE = 'BiocNeighbors', prebuilt_ptr, query, num_neighbors, force_variable_neighbors, num_threads, last_distance_only, report_index, report_distance)
}

generic_find_all <- function(prebuilt_ptr, chosen, thresholds, num_threads, report_index, report_distance) {
    .Call('_BiocNeighbors_generic_find_all', PACKAGE = 'BiocNeighbors', prebuilt_ptr, chosen, thresholds, num_threads, report_index, report_distance)
}

generic_query_all <- function(prebuilt_ptr, query, thresholds, num_threads, report_index, report_distance) {
    .Call('_BiocNeighbors_generic_query_all', PACKAGE = 'BiocNeighbors', prebuilt_ptr, query, thresholds, num_threads, report_index, report_distance)
}

hnsw_builder <- function(nlinks, ef_construct, ef_search, distance) {
    .Call('_BiocNeighbors_hnsw_builder', PACKAGE = 'BiocNeighbors', nlinks, ef_construct, ef_search, distance)
}

kmknn_builder <- function(distance) {
    .Call('_BiocNeighbors_kmknn_builder', PACKAGE = 'BiocNeighbors', distance)
}

vptree_builder <- function(distance) {
    .Call('_BiocNeighbors_vptree_builder', PACKAGE = 'BiocNeighbors', distance)
}

